# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSurface
from .zwp_tablet_v2 import ZwpTabletV2


class ZwpTabletToolV2(Interface):
    """A physical tablet tool

    An object that represents a physical tool that has been, or is currently in
    use with a tablet in this seat. Each wp_tablet_tool object stays valid
    until the client destroys it; the compositor reuses the wp_tablet_tool
    object to indicate that the object's respective physical tool has come into
    proximity of a tablet again.

    A wp_tablet_tool object's relation to a physical tool depends on the
    tablet's ability to report serial numbers. If the tablet supports this
    capability, then the object represents a specific physical tool and can be
    identified even when used on multiple tablets.

    A tablet tool has a number of static characteristics, e.g. tool type,
    hardware_serial and capabilities. These capabilities are sent in an event
    sequence after the wp_tablet_seat.tool_added event before any actual events
    from this tool. This initial event sequence is terminated by a
    wp_tablet_tool.done event.

    Tablet tool events are grouped by wp_tablet_tool.frame events. Any events
    received before a wp_tablet_tool.frame event should be considered part of
    the same hardware state change.
    """

    name = "zwp_tablet_tool_v2"
    version = 1

    class type(enum.IntEnum):
        pen = 0x140
        eraser = 0x141
        brush = 0x142
        pencil = 0x143
        airbrush = 0x144
        finger = 0x145
        mouse = 0x146
        lens = 0x147

    class capability(enum.IntEnum):
        tilt = 1
        pressure = 2
        distance = 3
        rotation = 4
        slider = 5
        wheel = 6

    class button_state(enum.IntEnum):
        released = 0
        pressed = 1

    class error(enum.IntEnum):
        role = 0


class ZwpTabletToolV2Proxy(Proxy):
    interface = ZwpTabletToolV2

    @ZwpTabletToolV2.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface, nullable=True),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_cursor(self, serial, surface, hotspot_x, hotspot_y):
        """Set the tablet tool's surface

        Sets the surface of the cursor used for this tool on the given tablet.
        This request only takes effect if the tool is in proximity of one of
        the requesting client's surfaces or the surface parameter is the
        current pointer surface. If there was a previous surface set with this
        request it is replaced. If surface is NULL, the cursor image is hidden.

        The parameters hotspot_x and hotspot_y define the position of the
        pointer surface relative to the pointer location. Its top-left corner
        is always at (x, y) - (hotspot_x, hotspot_y), where (x, y) are the
        coordinates of the pointer location, in surface-local coordinates.

        On surface.attach requests to the pointer surface, hotspot_x and
        hotspot_y are decremented by the x and y parameters passed to the
        request. Attach must be confirmed by :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` as usual.

        The hotspot can also be updated by passing the currently set pointer
        surface to this request with new values for hotspot_x and hotspot_y.

        The current and pending input regions of the
        :class:`~pywayland.protocol.wayland.WlSurface` are cleared, and
        :func:`WlSurface.set_input_region()
        <pywayland.protocol.wayland.WlSurface.set_input_region>` is ignored
        until the :class:`~pywayland.protocol.wayland.WlSurface` is no longer
        used as the cursor. When the use as a cursor ends, the current and
        pending input regions become undefined, and the
        :class:`~pywayland.protocol.wayland.WlSurface` is unmapped.

        This request gives the surface the role of a wp_tablet_tool cursor. A
        surface may only ever be used as the cursor surface for one
        wp_tablet_tool. If the surface already has another role or has
        previously been used as cursor surface for a different tool, a protocol
        error is raised.

        :param serial:
            serial of the enter event
        :type serial:
            `ArgumentType.Uint`
        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface` or `None`
        :param hotspot_x:
            surface-local x coordinate
        :type hotspot_x:
            `ArgumentType.Int`
        :param hotspot_y:
            surface-local y coordinate
        :type hotspot_y:
            `ArgumentType.Int`
        """
        self._marshal(0, serial, surface, hotspot_x, hotspot_y)

    @ZwpTabletToolV2.request()
    def destroy(self):
        """Destroy the tool object

        This destroys the client's resource for this tool object.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletToolV2Resource(Resource):
    interface = ZwpTabletToolV2

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def type(self, tool_type):
        """Tool type

        The tool type is the high-level type of the tool and usually decides
        the interaction expected from this tool.

        This event is sent in the initial burst of events before the
        wp_tablet_tool.done event.

        :param tool_type:
            the physical tool type
        :type tool_type:
            `ArgumentType.Uint`
        """
        self._post_event(0, tool_type)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def hardware_serial(self, hardware_serial_hi, hardware_serial_lo):
        """Unique hardware serial number of the tool

        If the physical tool can be identified by a unique 64-bit serial
        number, this event notifies the client of this serial number.

        If multiple tablets are available in the same seat and the tool is
        uniquely identifiable by the serial number, that tool may move between
        tablets.

        Otherwise, if the tool has no serial number and this event is missing,
        the tool is tied to the tablet it first comes into proximity with. Even
        if the physical tool is used on multiple tablets, separate
        wp_tablet_tool objects will be created, one per tablet.

        This event is sent in the initial burst of events before the
        wp_tablet_tool.done event.

        :param hardware_serial_hi:
            the unique serial number of the tool, most significant bits
        :type hardware_serial_hi:
            `ArgumentType.Uint`
        :param hardware_serial_lo:
            the unique serial number of the tool, least significant bits
        :type hardware_serial_lo:
            `ArgumentType.Uint`
        """
        self._post_event(1, hardware_serial_hi, hardware_serial_lo)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def hardware_id_wacom(self, hardware_id_hi, hardware_id_lo):
        """Hardware id notification in wacom's format

        This event notifies the client of a hardware id available on this tool.

        The hardware id is a device-specific 64-bit id that provides extra
        information about the tool in use, beyond the wl_tool.type enumeration.
        The format of the id is specific to tablets made by Wacom Inc. For
        example, the hardware id of a Wacom Grip Pen (a stylus) is 0x802.

        This event is sent in the initial burst of events before the
        wp_tablet_tool.done event.

        :param hardware_id_hi:
            the hardware id, most significant bits
        :type hardware_id_hi:
            `ArgumentType.Uint`
        :param hardware_id_lo:
            the hardware id, least significant bits
        :type hardware_id_lo:
            `ArgumentType.Uint`
        """
        self._post_event(2, hardware_id_hi, hardware_id_lo)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def capability(self, capability):
        """Tool capability notification

        This event notifies the client of any capabilities of this tool, beyond
        the main set of x/y axes and tip up/down detection.

        One event is sent for each extra capability available on this tool.

        This event is sent in the initial burst of events before the
        wp_tablet_tool.done event.

        :param capability:
            the capability
        :type capability:
            `ArgumentType.Uint`
        """
        self._post_event(3, capability)

    @ZwpTabletToolV2.event()
    def done(self):
        """Tool description events sequence complete

        This event signals the end of the initial burst of descriptive events.
        A client may consider the static description of the tool to be complete
        and finalize initialization of the tool.
        """
        self._post_event(4)

    @ZwpTabletToolV2.event()
    def removed(self):
        """Tool removed

        This event is sent when the tool is removed from the system and will
        send no further events. Should the physical tool come back into
        proximity later, a new wp_tablet_tool object will be created.

        It is compositor-dependent when a tool is removed. A compositor may
        remove a tool on proximity out, tablet removal or any other reason. A
        compositor may also keep a tool alive until shutdown.

        If the tool is currently in proximity, a proximity_out event will be
        sent before the removed event. See wp_tablet_tool.proximity_out for the
        handling of any buttons logically down.

        When this event is received, the client must wp_tablet_tool.destroy the
        object.
        """
        self._post_event(5)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=ZwpTabletV2),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def proximity_in(self, serial, tablet, surface):
        """Proximity in event

        Notification that this tool is focused on a certain surface.

        This event can be received when the tool has moved from one surface to
        another, or when the tool has come back into proximity above the
        surface.

        If any button is logically down when the tool comes into proximity, the
        respective button event is sent after the proximity_in event but within
        the same frame as the proximity_in event.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param tablet:
            The tablet the tool is in proximity of
        :type tablet:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletV2`
        :param surface:
            The current surface the tablet tool is over
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._post_event(6, serial, tablet, surface)

    @ZwpTabletToolV2.event()
    def proximity_out(self):
        """Proximity out event

        Notification that this tool has either left proximity, or is no longer
        focused on a certain surface.

        When the tablet tool leaves proximity of the tablet, button release
        events are sent for each button that was held down at the time of
        leaving proximity. These events are sent before the proximity_out event
        but within the same wp_tablet.frame.

        If the tool stays within proximity of the tablet, but the focus changes
        from one surface to another, a button release event may not be sent
        until the button is actually released or the tool leaves the proximity
        of the tablet.
        """
        self._post_event(7)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def down(self, serial):
        """Tablet tool is making contact

        Sent whenever the tablet tool comes in contact with the surface of the
        tablet.

        If the tool is already in contact with the tablet when entering the
        input region, the client owning said region will receive a
        wp_tablet.proximity_in event, followed by a wp_tablet.down event and a
        wp_tablet.frame event.

        Note that this event describes logical contact, not physical contact.
        On some devices, a compositor may not consider a tool in logical
        contact until a minimum physical pressure threshold is exceeded.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        """
        self._post_event(8, serial)

    @ZwpTabletToolV2.event()
    def up(self):
        """Tablet tool is no longer making contact

        Sent whenever the tablet tool stops making contact with the surface of
        the tablet, or when the tablet tool moves out of the input region and
        the compositor grab (if any) is dismissed.

        If the tablet tool moves out of the input region while in contact with
        the surface of the tablet and the compositor does not have an ongoing
        grab on the surface, the client owning said region will receive a
        wp_tablet.up event, followed by a wp_tablet.proximity_out event and a
        wp_tablet.frame event. If the compositor has an ongoing grab on this
        device, this event sequence is sent whenever the grab is dismissed in
        the future.

        Note that this event describes logical contact, not physical contact.
        On some devices, a compositor may not consider a tool out of logical
        contact until physical pressure falls below a specific threshold.
        """
        self._post_event(9)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def motion(self, x, y):
        """Motion event

        Sent whenever a tablet tool moves.

        :param x:
            surface-local x coordinate
        :type x:
            `ArgumentType.Fixed`
        :param y:
            surface-local y coordinate
        :type y:
            `ArgumentType.Fixed`
        """
        self._post_event(10, x, y)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def pressure(self, pressure):
        """Pressure change event

        Sent whenever the pressure axis on a tool changes. The value of this
        event is normalized to a value between 0 and 65535.

        Note that pressure may be nonzero even when a tool is not in logical
        contact. See the down and up events for more details.

        :param pressure:
            The current pressure value
        :type pressure:
            `ArgumentType.Uint`
        """
        self._post_event(11, pressure)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def distance(self, distance):
        """Distance change event

        Sent whenever the distance axis on a tool changes. The value of this
        event is normalized to a value between 0 and 65535.

        Note that distance may be nonzero even when a tool is not in logical
        contact. See the down and up events for more details.

        :param distance:
            The current distance value
        :type distance:
            `ArgumentType.Uint`
        """
        self._post_event(12, distance)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def tilt(self, tilt_x, tilt_y):
        """Tilt change event

        Sent whenever one or both of the tilt axes on a tool change. Each tilt
        value is in degrees, relative to the z-axis of the tablet. The angle is
        positive when the top of a tool tilts along the positive x or y axis.

        :param tilt_x:
            The current value of the X tilt axis
        :type tilt_x:
            `ArgumentType.Fixed`
        :param tilt_y:
            The current value of the Y tilt axis
        :type tilt_y:
            `ArgumentType.Fixed`
        """
        self._post_event(13, tilt_x, tilt_y)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Fixed),
    )
    def rotation(self, degrees):
        """Z-rotation change event

        Sent whenever the z-rotation axis on the tool changes. The rotation
        value is in degrees clockwise from the tool's logical neutral position.

        :param degrees:
            The current rotation of the Z axis
        :type degrees:
            `ArgumentType.Fixed`
        """
        self._post_event(14, degrees)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Int),
    )
    def slider(self, position):
        """Slider position change event

        Sent whenever the slider position on the tool changes. The value is
        normalized between -65535 and 65535, with 0 as the logical neutral
        position of the slider.

        The slider is available on e.g. the Wacom Airbrush tool.

        :param position:
            The current position of slider
        :type position:
            `ArgumentType.Int`
        """
        self._post_event(15, position)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Int),
    )
    def wheel(self, degrees, clicks):
        """Wheel delta event

        Sent whenever the wheel on the tool emits an event. This event contains
        two values for the same axis change. The degrees value is in the same
        orientation as the :func:`WlPointer.vertical_scroll()
        <pywayland.protocol.wayland.WlPointer.vertical_scroll>` axis. The
        clicks value is in discrete logical clicks of the mouse wheel. This
        value may be zero if the movement of the wheel was less than one
        logical click.

        Clients should choose either value and avoid mixing degrees and clicks.
        The compositor may accumulate values smaller than a logical click and
        emulate click events when a certain threshold is met. Thus,
        wl_tablet_tool.wheel events with non-zero clicks values may have
        different degrees values.

        :param degrees:
            The wheel delta in degrees
        :type degrees:
            `ArgumentType.Fixed`
        :param clicks:
            The wheel delta in discrete clicks
        :type clicks:
            `ArgumentType.Int`
        """
        self._post_event(16, degrees, clicks)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def button(self, serial, button, state):
        """Button event

        Sent whenever a button on the tool is pressed or released.

        If a button is held down when the tool moves in or out of proximity,
        button events are generated by the compositor. See
        wp_tablet_tool.proximity_in and wp_tablet_tool.proximity_out for
        details.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param button:
            The button whose state has changed
        :type button:
            `ArgumentType.Uint`
        :param state:
            Whether the button was pressed or released
        :type state:
            `ArgumentType.Uint`
        """
        self._post_event(17, serial, button, state)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def frame(self, time):
        """Frame event

        Marks the end of a series of axis and/or button updates from the
        tablet. The Wayland protocol requires axis updates to be sent
        sequentially, however all events within a frame should be considered
        one hardware event.

        :param time:
            The time of the event with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        """
        self._post_event(18, time)


class ZwpTabletToolV2Global(Global):
    interface = ZwpTabletToolV2


ZwpTabletToolV2._gen_c()
ZwpTabletToolV2.proxy_class = ZwpTabletToolV2Proxy
ZwpTabletToolV2.resource_class = ZwpTabletToolV2Resource
ZwpTabletToolV2.global_class = ZwpTabletToolV2Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Callable

from pywayland import ffi, lib
from pywayland.utils import wl_container_of


# void (*wl_notify_func_t)(struct wl_listener *listener, void *data);
@ffi.def_extern()
def notify_func(listener_ptr, data):
    container = wl_container_of(
        listener_ptr, "struct wl_listener_container *", "destroy_listener"
    )
    listener = ffi.from_handle(container.handle)

    if listener._signal is not None and listener._signal._data_wrapper is not None:
        data = listener._signal._data_wrapper(data)

    callback = listener._notify
    callback(listener, data)


class Listener:
    """A single listener for Wayland signals

    Provides the means to listen for `wl_listener` signal notifications.  Many
    Wayland objects use `wl_listener` for notification of significant events
    like object destruction.

    Clients should create :class:`Listener` objects manually and can register
    them as listeners to objects destroy events using the object's
    ``.add_destroy_listener()`` method.  A listener can only listen to one
    signal at a time.

    :param function: callback function for the Listener
    :type function: callable
    """

    container: "ffi.ListenerContainerCData"

    def __init__(self, function: Callable) -> None:
        self._handle = ffi.new_handle(self)

        # we need a way to get this Python object from the `struct
        # wl_listener*`, so we put the pointer in a container struct that
        # contains both the wl_listener and a pointer to our ffi handle
        self.container = ffi.new("struct wl_listener_container *")  # type: ignore[assignment]
        self.container.handle = self._handle

        self._ptr = ffi.addressof(self.container.destroy_listener)
        self._ptr.notify = lib.notify_func
        self._notify = function
        self._signal = None

    def remove(self) -> None:
        """Remove the listener"""
        if self._ptr.link != ffi.NULL:
            lib.wl_list_remove(ffi.addressof(self._ptr.link))
            self.link = None


class Signal:
    """A source of a type of observable event

    Signals are recognized points where significant events can be observed.
    Compositors as well as the server can provide signals. Observers are
    wl_listener's that are added through #wl_signal_add. Signals are emitted
    using #wl_signal_emit, which will invoke all listeners until that listener
    is removed by wl_list_remove() (or whenever the signal is destroyed).
    """

    def __init__(self, *, ptr=None, data_wrapper=None):
        if ptr is None:
            self._ptr = ffi.new("struct wl_listener *")
            lib.wl_signal_init(self._ptr)
        else:
            self._ptr = ptr

        self._data_wrapper = data_wrapper
        self._link = []

    def add(self, listener):
        """Add the specified listener to this signal

        :param listener: The listener to add
        :type listener: :class:`Listener`
        """
        # we are creating a reference loop, as the listener resolves the data
        # wrapper function, and the signal keeps the listener alive
        listener._signal = self
        self._link.append(listener)

        lib.wl_signal_add(self._ptr, listener._ptr)

    def emit(self, data=None):
        """Emits this signal, notifying all registered listeners

        :param data: The data that will be emitted with the signal
        """
        if data is not None:
            data_ptr = ffi.new_handle(data)
        else:
            data_ptr = ffi.NULL
        lib.wl_signal_emit(self._ptr, data_ptr)



# This file has been autogenerated by the pywayland scanner

# Copyright © 2013-2016 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class WpViewport(Interface):
    """Crop and scale interface to a :class:`~pywayland.protocol.wayland.WlSurface`

    An additional interface to a :class:`~pywayland.protocol.wayland.WlSurface`
    object, which allows the client to specify the cropping and scaling of the
    surface contents.

    This interface works with two concepts: the source rectangle (src_x, src_y,
    src_width, src_height), and the destination size (dst_width, dst_height).
    The contents of the source rectangle are scaled to the destination size,
    and content outside the source rectangle is ignored. This state is double-
    buffered, and is applied on the next :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>`.

    The two parts of crop and scale state are independent: the source
    rectangle, and the destination size. Initially both are unset, that is, no
    scaling is applied. The whole of the current
    :class:`~pywayland.protocol.wayland.WlBuffer` is used as the source, and
    the surface size is as defined in :func:`WlSurface.attach()
    <pywayland.protocol.wayland.WlSurface.attach>`.

    If the destination size is set, it causes the surface size to become
    dst_width, dst_height. The source (rectangle) is scaled to exactly this
    size. This overrides whatever the attached
    :class:`~pywayland.protocol.wayland.WlBuffer` size is, unless the
    :class:`~pywayland.protocol.wayland.WlBuffer` is NULL. If the
    :class:`~pywayland.protocol.wayland.WlBuffer` is NULL, the surface has no
    content and therefore no size. Otherwise, the size is always at least 1x1
    in surface local coordinates.

    If the source rectangle is set, it defines what area of the
    :class:`~pywayland.protocol.wayland.WlBuffer` is taken as the source. If
    the source rectangle is set and the destination size is not set, then
    src_width and src_height must be integers, and the surface size becomes the
    source rectangle size. This results in cropping without scaling. If
    src_width or src_height are not integers and destination size is not set,
    the bad_size protocol error is raised when the surface state is applied.

    The coordinate transformations from buffer pixel coordinates up to the
    surface-local coordinates happen in the following order:   1.
    buffer_transform (:func:`WlSurface.set_buffer_transform()
    <pywayland.protocol.wayland.WlSurface.set_buffer_transform>`)   2.
    buffer_scale (:func:`WlSurface.set_buffer_scale()
    <pywayland.protocol.wayland.WlSurface.set_buffer_scale>`)   3. crop and
    scale (:class:`WpViewport`.set*) This means, that the source rectangle
    coordinates of crop and scale are given in the coordinates after the buffer
    transform and scale, i.e. in the coordinates that would be the surface-
    local coordinates if the crop and scale was not applied.

    If src_x or src_y are negative, the bad_value protocol error is raised.
    Otherwise, if the source rectangle is partially or completely outside of
    the non-NULL :class:`~pywayland.protocol.wayland.WlBuffer`, then the
    out_of_buffer protocol error is raised when the surface state is applied. A
    NULL :class:`~pywayland.protocol.wayland.WlBuffer` does not raise the
    out_of_buffer error.

    The x, y arguments of :func:`WlSurface.attach()
    <pywayland.protocol.wayland.WlSurface.attach>` are applied as normal to the
    surface. They indicate how many pixels to remove from the surface size from
    the left and the top. In other words, they are still in the surface-local
    coordinate system, just like dst_width and dst_height are.

    If the :class:`~pywayland.protocol.wayland.WlSurface` associated with the
    :class:`WpViewport` is destroyed, all :class:`WpViewport` requests except
    'destroy' raise the protocol error no_surface.

    If the :class:`WpViewport` object is destroyed, the crop and scale state is
    removed from the :class:`~pywayland.protocol.wayland.WlSurface`. The change
    will be applied on the next :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>`.
    """

    name = "wp_viewport"
    version = 1

    class error(enum.IntEnum):
        bad_value = 0
        bad_size = 1
        out_of_buffer = 2
        no_surface = 3


class WpViewportProxy(Proxy):
    interface = WpViewport

    @WpViewport.request()
    def destroy(self):
        """Remove scaling and cropping from the surface

        The associated wl_surface's crop and scale state is removed. The change
        is applied on the next :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.
        """
        self._marshal(0)
        self._destroy()

    @WpViewport.request(
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def set_source(self, x, y, width, height):
        """Set the source rectangle for cropping

        Set the source rectangle of the associated
        :class:`~pywayland.protocol.wayland.WlSurface`. See :class:`WpViewport`
        for the description, and relation to the
        :class:`~pywayland.protocol.wayland.WlBuffer` size.

        If all of x, y, width and height are -1.0, the source rectangle is
        unset instead. Any other set of values where width or height are zero
        or negative, or x or y are negative, raise the bad_value protocol
        error.

        The crop and scale state is double-buffered state, and will be applied
        on the next :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        :param x:
            source rectangle x
        :type x:
            `ArgumentType.Fixed`
        :param y:
            source rectangle y
        :type y:
            `ArgumentType.Fixed`
        :param width:
            source rectangle width
        :type width:
            `ArgumentType.Fixed`
        :param height:
            source rectangle height
        :type height:
            `ArgumentType.Fixed`
        """
        self._marshal(1, x, y, width, height)

    @WpViewport.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_destination(self, width, height):
        """Set the surface size for scaling

        Set the destination size of the associated
        :class:`~pywayland.protocol.wayland.WlSurface`. See :class:`WpViewport`
        for the description, and relation to the
        :class:`~pywayland.protocol.wayland.WlBuffer` size.

        If width is -1 and height is -1, the destination size is unset instead.
        Any other pair of values for width and height that contains zero or
        negative values raises the bad_value protocol error.

        The crop and scale state is double-buffered state, and will be applied
        on the next :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`.

        :param width:
            surface width
        :type width:
            `ArgumentType.Int`
        :param height:
            surface height
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(2, width, height)


class WpViewportResource(Resource):
    interface = WpViewport


class WpViewportGlobal(Global):
    interface = WpViewport


WpViewport._gen_c()
WpViewport.proxy_class = WpViewportProxy
WpViewport.resource_class = WpViewportResource
WpViewport.global_class = WpViewportGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
import logging
from typing import Any, Optional, Tuple

from pywayland import ffi, lib
from pywayland.utils import ensure_valid
from .display import Display
from .listener import Listener


def _client_destroy(display: Display, cdata: "ffi.ClientCData") -> None:
    # do nothing if the display is already destroyed
    if display.destroyed:
        logging.error("Display destroyed before client")
        return

    lib.wl_client_destroy(cdata)


class Client:
    """Create a client for the given file descriptor

    Given a file descriptor corresponding to one end of a socket, create a
    client struct and add the new client to the compositors client list.  At
    that point, the client is initialized and ready to run, as if the client
    had connected to the servers listening socket.

    The other end of the socket can be passed to
    :meth:`~pywayland.client.Display.connect()` on the client side or used with
    the WAYLAND_SOCKET environment variable on the client side.

    :param display: The display object
    :type display: :class:`Display`
    :param fd: The file descriptor for the socket to the client
    :type fd: `int`
    """

    def __init__(self, display: Display, fd: int) -> None:
        if display.destroyed:
            raise ValueError("Display has been destroyed")

        ptr = lib.wl_client_create(display._ptr, fd)

        destructor = functools.partial(_client_destroy, display)
        self._ptr: Optional["ffi.ClientCData"] = ffi.gc(ptr, destructor)
        self._display: Optional[Display] = display

    def destroy(self) -> None:
        """Destroy the client"""
        if self._ptr is not None:
            ffi.release(self._ptr)
            self._ptr = None
            self._display = None

    @ensure_valid
    def flush(self) -> None:
        """Flush pending events to the client

        Events sent to clients are queued in a buffer and written to the socket
        later - typically when the compositor has handled all requests and goes
        back to block in the event loop.  This function flushes all queued up
        events for a client immediately.
        """
        assert self._ptr is not None
        lib.wl_client_flush(self._ptr)

    @ensure_valid
    def get_credentials(self) -> Tuple[int, int, int]:
        """Return Unix credentials for the client.

        This function returns the process ID, the user ID and the group ID for the given
        client. The credentials come from getsockopt() with SO_PEERCRED, on the client
        socket fd.
        """
        assert self._ptr is not None

        pid = ffi.new("pid_t *")
        uid = ffi.new("uid_t *")
        gid = ffi.new("gid_t *")
        lib.wl_client_get_credentials(self._ptr, pid, uid, gid)
        return pid[0], uid[0], gid[0]

    @ensure_valid
    def add_destroy_listener(self, listener: Listener) -> None:
        """Add a listener for the destroy signal

        :param listener: The listener object
        :type listener: :class:`~pywayland.server.Listener`
        """
        assert self._ptr is not None
        lib.wl_client_add_destroy_listener(self._ptr, listener._ptr)

    @ensure_valid
    def get_object(self, object_id: int) -> Any:
        """Look up an object in the client name space

        This looks up an object in the client object name space by its object
        ID.

        :param object_id: The object id
        :type object_id: `int`
        :returns:
            The object, or ``None`` if there is not object for the given ID
        """
        assert self._ptr is not None

        res_ptr = lib.wl_client_get_object(self._ptr, object_id)
        # If the object doesn't exist, this returns NULL, and asking for
        # forgiveness doesn't work, becuase it will seg fault
        if res_ptr == ffi.NULL:
            return None

        resource_handle = lib.wl_resource_get_user_data(res_ptr)
        return ffi.from_handle(resource_handle)



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_surface import WlSurface


class WlPointer(Interface):
    """Pointer input device

    The :class:`WlPointer` interface represents one or more input devices, such
    as mice, which control the pointer location and pointer_focus of a seat.

    The :class:`WlPointer` interface generates motion, enter and leave events
    for the surfaces that the pointer is located over, and button and axis
    events for button presses, button releases and scrolling.
    """

    name = "wl_pointer"
    version = 7

    class error(enum.IntEnum):
        role = 0

    class button_state(enum.IntEnum):
        released = 0
        pressed = 1

    class axis(enum.IntEnum):
        vertical_scroll = 0
        horizontal_scroll = 1

    class axis_source(enum.IntEnum):
        wheel = 0
        finger = 1
        continuous = 2
        wheel_tilt = 3


class WlPointerProxy(Proxy):
    interface = WlPointer

    @WlPointer.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface, nullable=True),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_cursor(self, serial, surface, hotspot_x, hotspot_y):
        """Set the pointer surface

        Set the pointer surface, i.e., the surface that contains the pointer
        image (cursor). This request gives the surface the role of a cursor. If
        the surface already has another role, it raises a protocol error.

        The cursor actually changes only if the pointer focus for this device
        is one of the requesting client's surfaces or the surface parameter is
        the current pointer surface. If there was a previous surface set with
        this request it is replaced. If surface is NULL, the pointer image is
        hidden.

        The parameters hotspot_x and hotspot_y define the position of the
        pointer surface relative to the pointer location. Its top-left corner
        is always at (x, y) - (hotspot_x, hotspot_y), where (x, y) are the
        coordinates of the pointer location, in surface-local coordinates.

        On surface.attach requests to the pointer surface, hotspot_x and
        hotspot_y are decremented by the x and y parameters passed to the
        request. Attach must be confirmed by :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` as usual.

        The hotspot can also be updated by passing the currently set pointer
        surface to this request with new values for hotspot_x and hotspot_y.

        The current and pending input regions of the
        :class:`~pywayland.protocol.wayland.WlSurface` are cleared, and
        :func:`WlSurface.set_input_region()
        <pywayland.protocol.wayland.WlSurface.set_input_region>` is ignored
        until the :class:`~pywayland.protocol.wayland.WlSurface` is no longer
        used as the cursor. When the use as a cursor ends, the current and
        pending input regions become undefined, and the
        :class:`~pywayland.protocol.wayland.WlSurface` is unmapped.

        :param serial:
            serial number of the enter event
        :type serial:
            `ArgumentType.Uint`
        :param surface:
            pointer surface
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface` or `None`
        :param hotspot_x:
            surface-local x coordinate
        :type hotspot_x:
            `ArgumentType.Int`
        :param hotspot_y:
            surface-local y coordinate
        :type hotspot_y:
            `ArgumentType.Int`
        """
        self._marshal(0, serial, surface, hotspot_x, hotspot_y)

    @WlPointer.request(version=3)
    def release(self):
        """Release the pointer object

        Using this request a client can tell the server that it is not going to
        use the pointer object anymore.

        This request destroys the pointer proxy object, so clients must not
        call wl_pointer_destroy() after using this request.
        """
        self._marshal(1)
        self._destroy()


class WlPointerResource(Resource):
    interface = WlPointer

    @WlPointer.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def enter(self, serial, surface, surface_x, surface_y):
        """Enter event

        Notification that this seat's pointer is focused on a certain surface.

        When a seat's focus enters a surface, the pointer image is undefined
        and a client should respond to this event by setting an appropriate
        pointer image with the set_cursor request.

        :param serial:
            serial number of the enter event
        :type serial:
            `ArgumentType.Uint`
        :param surface:
            surface entered by the pointer
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param surface_x:
            surface-local x coordinate
        :type surface_x:
            `ArgumentType.Fixed`
        :param surface_y:
            surface-local y coordinate
        :type surface_y:
            `ArgumentType.Fixed`
        """
        self._post_event(0, serial, surface, surface_x, surface_y)

    @WlPointer.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def leave(self, serial, surface):
        """Leave event

        Notification that this seat's pointer is no longer focused on a certain
        surface.

        The leave notification is sent before the enter notification for the
        new focus.

        :param serial:
            serial number of the leave event
        :type serial:
            `ArgumentType.Uint`
        :param surface:
            surface left by the pointer
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._post_event(1, serial, surface)

    @WlPointer.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def motion(self, time, surface_x, surface_y):
        """Pointer motion event

        Notification of pointer location change. The arguments surface_x and
        surface_y are the location relative to the focused surface.

        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param surface_x:
            surface-local x coordinate
        :type surface_x:
            `ArgumentType.Fixed`
        :param surface_y:
            surface-local y coordinate
        :type surface_y:
            `ArgumentType.Fixed`
        """
        self._post_event(2, time, surface_x, surface_y)

    @WlPointer.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def button(self, serial, time, button, state):
        """Pointer button event

        Mouse button click and release notifications.

        The location of the click is given by the last motion or enter event.
        The time argument is a timestamp with millisecond granularity, with an
        undefined base.

        The button is a button code as defined in the Linux kernel's
        linux/input-event-codes.h header file, e.g. BTN_LEFT.

        Any 16-bit button code value is reserved for future additions to the
        kernel's event code list. All other button codes above 0xFFFF are
        currently undefined but may be used in future versions of this
        protocol.

        :param serial:
            serial number of the button event
        :type serial:
            `ArgumentType.Uint`
        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param button:
            button that produced the event
        :type button:
            `ArgumentType.Uint`
        :param state:
            physical state of the button
        :type state:
            `ArgumentType.Uint`
        """
        self._post_event(3, serial, time, button, state)

    @WlPointer.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Fixed),
    )
    def axis(self, time, axis, value):
        """Axis event

        Scroll and other axis notifications.

        For scroll events (vertical and horizontal scroll axes), the value
        parameter is the length of a vector along the specified axis in a
        coordinate space identical to those of motion events, representing a
        relative movement along the specified axis.

        For devices that support movements non-parallel to axes multiple axis
        events will be emitted.

        When applicable, for example for touch pads, the server can choose to
        emit scroll events where the motion vector is equivalent to a motion
        event vector.

        When applicable, a client can transform its content relative to the
        scroll distance.

        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param axis:
            axis type
        :type axis:
            `ArgumentType.Uint`
        :param value:
            length of vector in surface-local coordinate space
        :type value:
            `ArgumentType.Fixed`
        """
        self._post_event(4, time, axis, value)

    @WlPointer.event(version=5)
    def frame(self):
        """End of a pointer event sequence

        Indicates the end of a set of events that logically belong together. A
        client is expected to accumulate the data in all events within the
        frame before proceeding.

        All :class:`WlPointer` events before a :func:`WlPointer.frame()` event
        belong logically together. For example, in a diagonal scroll motion the
        compositor will send an optional :func:`WlPointer.axis_source()` event,
        two :func:`WlPointer.axis()` events (horizontal and vertical) and
        finally a :func:`WlPointer.frame()` event. The client may use this
        information to calculate a diagonal vector for scrolling.

        When multiple :func:`WlPointer.axis()` events occur within the same
        frame, the motion vector is the combined motion of all events. When a
        :func:`WlPointer.axis()` and a :func:`WlPointer.axis_stop()` event
        occur within the same frame, this indicates that axis movement in one
        axis has stopped but continues in the other axis. When multiple
        :func:`WlPointer.axis_stop()` events occur within the same frame, this
        indicates that these axes stopped in the same instance.

        A :func:`WlPointer.frame()` event is sent for every logical event
        group, even if the group only contains a single :class:`WlPointer`
        event. Specifically, a client may get a sequence: motion, frame,
        button, frame, axis, frame, axis_stop, frame.

        The :func:`WlPointer.enter()` and :func:`WlPointer.leave()` events are
        logical events generated by the compositor and not the hardware. These
        events are also grouped by a :func:`WlPointer.frame()`. When a pointer
        moves from one surface to another, a compositor should group the
        :func:`WlPointer.leave()` event within the same
        :func:`WlPointer.frame()`. However, a client must not rely on
        :func:`WlPointer.leave()` and :func:`WlPointer.enter()` being in the
        same :func:`WlPointer.frame()`. Compositor-specific policies may
        require the :func:`WlPointer.leave()` and :func:`WlPointer.enter()`
        event being split across multiple :func:`WlPointer.frame()` groups.
        """
        self._post_event(5)

    @WlPointer.event(
        Argument(ArgumentType.Uint),
        version=5,
    )
    def axis_source(self, axis_source):
        """Axis source event

        Source information for scroll and other axes.

        This event does not occur on its own. It is sent before a
        :func:`WlPointer.frame()` event and carries the source information for
        all events within that frame.

        The source specifies how this event was generated. If the source is
        :func:`WlPointer.axis_source()`.finger, a :func:`WlPointer.axis_stop()`
        event will be sent when the user lifts the finger off the device.

        If the source is :func:`WlPointer.axis_source()`.wheel,
        :func:`WlPointer.axis_source()`.wheel_tilt or
        :func:`WlPointer.axis_source()`.continuous, a
        :func:`WlPointer.axis_stop()` event may or may not be sent. Whether a
        compositor sends an axis_stop event for these sources is hardware-
        specific and implementation-dependent; clients must not rely on
        receiving an axis_stop event for these scroll sources and should treat
        scroll sequences from these scroll sources as unterminated by default.

        This event is optional. If the source is unknown for a particular axis
        event sequence, no event is sent. Only one
        :func:`WlPointer.axis_source()` event is permitted per frame.

        The order of :func:`WlPointer.axis_discrete()` and
        :func:`WlPointer.axis_source()` is not guaranteed.

        :param axis_source:
            source of the axis event
        :type axis_source:
            `ArgumentType.Uint`
        """
        self._post_event(6, axis_source)

    @WlPointer.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        version=5,
    )
    def axis_stop(self, time, axis):
        """Axis stop event

        Stop notification for scroll and other axes.

        For some :func:`WlPointer.axis_source()` types, a
        :func:`WlPointer.axis_stop()` event is sent to notify a client that the
        axis sequence has terminated. This enables the client to implement
        kinetic scrolling. See the :func:`WlPointer.axis_source()`
        documentation for information on when this event may be generated.

        Any :func:`WlPointer.axis()` events with the same axis_source after
        this event should be considered as the start of a new axis motion.

        The timestamp is to be interpreted identical to the timestamp in the
        :func:`WlPointer.axis()` event. The timestamp value may be the same as
        a preceding :func:`WlPointer.axis()` event.

        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param axis:
            the axis stopped with this event
        :type axis:
            `ArgumentType.Uint`
        """
        self._post_event(7, time, axis)

    @WlPointer.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Int),
        version=5,
    )
    def axis_discrete(self, axis, discrete):
        """Axis click event

        Discrete step information for scroll and other axes.

        This event carries the axis value of the :func:`WlPointer.axis()` event
        in discrete steps (e.g. mouse wheel clicks).

        This event does not occur on its own, it is coupled with a
        :func:`WlPointer.axis()` event that represents this axis value on a
        continuous scale. The protocol guarantees that each axis_discrete event
        is always followed by exactly one axis event with the same axis number
        within the same :func:`WlPointer.frame()`. Note that the protocol
        allows for other events to occur between the axis_discrete and its
        coupled axis event, including other axis_discrete or axis events.

        This event is optional; continuous scrolling devices like two-finger
        scrolling on touchpads do not have discrete steps and do not generate
        this event.

        The discrete value carries the directional information. e.g. a value of
        -2 is two steps towards the negative direction of this axis.

        The axis number is identical to the axis number in the associated axis
        event.

        The order of :func:`WlPointer.axis_discrete()` and
        :func:`WlPointer.axis_source()` is not guaranteed.

        :param axis:
            axis type
        :type axis:
            `ArgumentType.Uint`
        :param discrete:
            number of steps
        :type discrete:
            `ArgumentType.Int`
        """
        self._post_event(8, axis, discrete)


class WlPointerGlobal(Global):
    interface = WlPointer


WlPointer._gen_c()
WlPointer.proxy_class = WlPointerProxy
WlPointer.resource_class = WlPointerResource
WlPointer.global_class = WlPointerGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright 2016 The Chromium Authors.
# Copyright 2017 Intel Corporation
# Copyright 2018 Collabora, Ltd
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSurface
from .zwp_linux_surface_synchronization_v1 import ZwpLinuxSurfaceSynchronizationV1


class ZwpLinuxExplicitSynchronizationV1(Interface):
    """Protocol for providing explicit synchronization

    This global is a factory interface, allowing clients to request explicit
    synchronization for buffers on a per-surface basis.

    See
    :class:`~pywayland.protocol.zwp_linux_explicit_synchronization_unstable_v1.ZwpLinuxSurfaceSynchronizationV1`
    for more information.

    This interface is derived from Chromium's
    zcr_linux_explicit_synchronization_v1.

    Warning! The protocol described in this file is experimental and backward
    incompatible changes may be made. Backward compatible changes may be added
    together with the corresponding interface version bump. Backward
    incompatible changes are done by bumping the version number in the protocol
    and interface names and resetting the interface version. Once the protocol
    is to be declared stable, the 'z' prefix and the version number in the
    protocol and interface names are removed and the interface version number
    is reset.
    """

    name = "zwp_linux_explicit_synchronization_v1"
    version = 2

    class error(enum.IntEnum):
        synchronization_exists = 0


class ZwpLinuxExplicitSynchronizationV1Proxy(Proxy):
    interface = ZwpLinuxExplicitSynchronizationV1

    @ZwpLinuxExplicitSynchronizationV1.request()
    def destroy(self):
        """Destroy explicit synchronization factory object

        Destroy this explicit synchronization factory object. Other objects,
        including
        :class:`~pywayland.protocol.zwp_linux_explicit_synchronization_unstable_v1.ZwpLinuxSurfaceSynchronizationV1`
        objects created by this factory, shall not be affected by this request.
        """
        self._marshal(0)
        self._destroy()

    @ZwpLinuxExplicitSynchronizationV1.request(
        Argument(ArgumentType.NewId, interface=ZwpLinuxSurfaceSynchronizationV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_synchronization(self, surface):
        """Extend surface interface for explicit synchronization

        Instantiate an interface extension for the given
        :class:`~pywayland.protocol.wayland.WlSurface` to provide explicit
        synchronization.

        If the given :class:`~pywayland.protocol.wayland.WlSurface` already has
        an explicit synchronization object associated, the
        synchronization_exists protocol error is raised.

        Graphics APIs, like EGL or Vulkan, that manage the buffer queue and
        commits of a :class:`~pywayland.protocol.wayland.WlSurface` themselves,
        are likely to be using this extension internally. If a client is using
        such an API for a :class:`~pywayland.protocol.wayland.WlSurface`, it
        should not directly use this extension on that surface, to avoid
        raising a synchronization_exists protocol error.

        :param surface:
            the surface
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.zwp_linux_explicit_synchronization_unstable_v1.ZwpLinuxSurfaceSynchronizationV1`
            -- the new synchronization interface id
        """
        id = self._marshal_constructor(1, ZwpLinuxSurfaceSynchronizationV1, surface)
        return id


class ZwpLinuxExplicitSynchronizationV1Resource(Resource):
    interface = ZwpLinuxExplicitSynchronizationV1


class ZwpLinuxExplicitSynchronizationV1Global(Global):
    interface = ZwpLinuxExplicitSynchronizationV1


ZwpLinuxExplicitSynchronizationV1._gen_c()
ZwpLinuxExplicitSynchronizationV1.proxy_class = ZwpLinuxExplicitSynchronizationV1Proxy
ZwpLinuxExplicitSynchronizationV1.resource_class = ZwpLinuxExplicitSynchronizationV1Resource
ZwpLinuxExplicitSynchronizationV1.global_class = ZwpLinuxExplicitSynchronizationV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2017 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class ZwpInputTimestampsV1(Interface):
    """Context object for input timestamps

    Provides high-resolution timestamp events for a set of subscribed input
    events. The set of subscribed input events is determined by the
    :class:`~pywayland.protocol.input_timestamps_unstable_v1.ZwpInputTimestampsManagerV1`
    request used to create this object.
    """

    name = "zwp_input_timestamps_v1"
    version = 1


class ZwpInputTimestampsV1Proxy(Proxy):
    interface = ZwpInputTimestampsV1

    @ZwpInputTimestampsV1.request()
    def destroy(self):
        """Destroy the input timestamps object

        Informs the server that the client will no longer be using this
        protocol object. After the server processes the request, no more
        timestamp events will be emitted.
        """
        self._marshal(0)
        self._destroy()


class ZwpInputTimestampsV1Resource(Resource):
    interface = ZwpInputTimestampsV1

    @ZwpInputTimestampsV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def timestamp(self, tv_sec_hi, tv_sec_lo, tv_nsec):
        """High-resolution timestamp event

        The timestamp event is associated with the first subsequent input event
        carrying a timestamp which belongs to the set of input events this
        object is subscribed to.

        The timestamp provided by this event is a high-resolution version of
        the timestamp argument of the associated input event. The provided
        timestamp is in the same clock domain and is at least as accurate as
        the associated input event timestamp.

        The timestamp is expressed as tv_sec_hi, tv_sec_lo, tv_nsec triples,
        each component being an unsigned 32-bit value. Whole seconds are in
        tv_sec which is a 64-bit value combined from tv_sec_hi and tv_sec_lo,
        and the additional fractional part in tv_nsec as nanoseconds. Hence,
        for valid timestamps tv_nsec must be in [0, 999999999].

        :param tv_sec_hi:
            high 32 bits of the seconds part of the timestamp
        :type tv_sec_hi:
            `ArgumentType.Uint`
        :param tv_sec_lo:
            low 32 bits of the seconds part of the timestamp
        :type tv_sec_lo:
            `ArgumentType.Uint`
        :param tv_nsec:
            nanoseconds part of the timestamp
        :type tv_nsec:
            `ArgumentType.Uint`
        """
        self._post_event(0, tv_sec_hi, tv_sec_lo, tv_nsec)


class ZwpInputTimestampsV1Global(Global):
    interface = ZwpInputTimestampsV1


ZwpInputTimestampsV1._gen_c()
ZwpInputTimestampsV1.proxy_class = ZwpInputTimestampsV1Proxy
ZwpInputTimestampsV1.resource_class = ZwpInputTimestampsV1Resource
ZwpInputTimestampsV1.global_class = ZwpInputTimestampsV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
# Copyright © 2015-2017 Samsung Electronics Co., Ltd
# Copyright © 2015-2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from .xdg_popup import XdgPopup  # noqa: F401
from .xdg_positioner import XdgPositioner  # noqa: F401
from .xdg_surface import XdgSurface  # noqa: F401
from .xdg_toplevel import XdgToplevel  # noqa: F401
from .xdg_wm_base import XdgWmBase  # noqa: F401



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from typing import Optional, Type, TYPE_CHECKING

from pywayland.scanner.argument import ArgumentType

if TYPE_CHECKING:
    from .interface import Interface  # noqa: F401


class classproperty:
    def __init__(self, f):
        self.f = f

    def __get__(self, obj, owner):
        return self.f(owner)


@dataclass(frozen=True)
class Argument:
    argument_type: ArgumentType
    nullable: bool = False
    interface: Optional[Type["Interface"]] = None

    @property
    def signature(self) -> str:
        if self.argument_type == ArgumentType.Int:
            base_signature = "i"
        elif self.argument_type == ArgumentType.Uint:
            base_signature = "u"
        elif self.argument_type == ArgumentType.Fixed:
            base_signature = "f"
        elif self.argument_type == ArgumentType.String:
            base_signature = "s"
        elif self.argument_type == ArgumentType.Object:
            base_signature = "o"
        elif self.argument_type == ArgumentType.NewId:
            if self.interface is None:
                base_signature = "sun"
            else:
                base_signature = "n"
        elif self.argument_type == ArgumentType.Array:
            base_signature = "a"
        elif self.argument_type == ArgumentType.FileDescriptor:
            base_signature = "h"

        if self.nullable:
            return "?" + base_signature
        return base_signature



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.server.eventloop import EventLoop
from pywayland.server.listener import Listener

import os
import signal
import time


class _GetCallback(object):
    def __init__(self):
        self.callback = None
        self.n_calls = 0


def _fd_callback(fd, mask, data):
    data.callback = fd
    data.n_calls += 1
    return 0


def _signal_callback(signal_number, data):
    data.callback = signal_number
    data.n_calls += 1
    return 1


def _timer_callback(data):
    data.n_calls += 1
    return 1


def test_event_loop_post_dispatch_check():
    callback = _GetCallback()
    event_loop = EventLoop()

    r, w = os.pipe()

    try:
        source = event_loop.add_fd(r, _fd_callback, EventLoop.FdMask.WL_EVENT_READABLE, callback)
        source.check()

        event_loop.dispatch(0)
        assert callback.callback == r
    finally:
        os.close(r)
        os.close(w)


def test_event_loop_signal():
    callback = _GetCallback()
    event_loop = EventLoop()

    event_loop.add_signal(signal.SIGUSR1, _signal_callback, callback)

    event_loop.dispatch(0)
    assert callback.callback is None

    os.kill(os.getpid(), signal.SIGUSR1)

    event_loop.dispatch(0)
    assert callback.callback == signal.SIGUSR1


def test_event_loop_multiple_same_signals():
    callback = _GetCallback()
    event_loop = EventLoop()

    signal_rm = event_loop.add_signal(signal.SIGUSR1, _signal_callback, callback)
    event_loop.add_signal(signal.SIGUSR1, _signal_callback, callback)

    event_loop.dispatch(0)
    assert callback.n_calls == 0

    # Check callback gets 2 calls
    for _ in range(5):
        callback.n_calls = 0
        os.kill(os.getpid(), signal.SIGUSR1)
        event_loop.dispatch(0)

        assert callback.n_calls == 2

    # Remove one of the signals
    signal_rm.remove()

    # Callback only gets call now
    for _ in range(5):
        callback.n_calls = 0
        os.kill(os.getpid(), signal.SIGUSR1)
        event_loop.dispatch(0)

        assert callback.n_calls == 1


def test_event_loop_timer():
    callback = _GetCallback()
    event_loop = EventLoop()

    source = event_loop.add_timer(_timer_callback, callback)
    source.timer_update(10)

    event_loop.dispatch(0)
    assert callback.n_calls == 0

    event_loop.dispatch(20)
    assert callback.n_calls == 1


def _timer_update_callback1(data):
    data.n_calls += 1
    data.source2.timer_update(1000)
    return 1


def _timer_update_callback2(data):
    data.n_calls += 1
    data.source1.timer_update(1000)
    return 1


def test_event_loop_timer_updates():
    callback = _GetCallback()
    event_loop = EventLoop()

    source1 = event_loop.add_timer(_timer_update_callback1, callback)
    source1.timer_update(10)

    source2 = event_loop.add_timer(_timer_update_callback2, callback)
    source2.timer_update(10)

    callback.source1 = source1
    callback.source2 = source2

    assert callback.n_calls == 0

    # Wait 15 ms, so both timers should be called when we dispatch
    time.sleep(0.015)

    # This should take < 1 sec
    start_time = time.time()
    event_loop.dispatch(20)
    end_time = time.time()

    assert callback.n_calls == 2

    assert end_time - start_time < 1


def _destroy_notify_a(listener, data):
    global a
    a = True


def _destroy_notify_b(listener, data):
    global b
    b = True


def test_event_loop_destroy():
    global a, b
    a = False
    b = False

    event_loop = EventLoop()
    listener_a = Listener(_destroy_notify_a)
    listener_b = Listener(_destroy_notify_b)

    event_loop.add_destroy_listener(listener_a)
    event_loop.add_destroy_listener(listener_b)

    listener_a.remove()

    event_loop.destroy()

    import gc
    gc.collect()

    assert a is False
    assert b is True



# This file has been autogenerated by the pywayland scanner

# Copyright © 2012, 2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSurface
from .zwp_input_panel_surface_v1 import ZwpInputPanelSurfaceV1


class ZwpInputPanelV1(Interface):
    """Interface for implementing keyboards

    Only one client can bind this interface at a time.
    """

    name = "zwp_input_panel_v1"
    version = 1


class ZwpInputPanelV1Proxy(Proxy):
    interface = ZwpInputPanelV1

    @ZwpInputPanelV1.request(
        Argument(ArgumentType.NewId, interface=ZwpInputPanelSurfaceV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_input_panel_surface(self, surface):
        """get_input_panel_surface

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.input_method_unstable_v1.ZwpInputPanelSurfaceV1`
        """
        id = self._marshal_constructor(0, ZwpInputPanelSurfaceV1, surface)
        return id


class ZwpInputPanelV1Resource(Resource):
    interface = ZwpInputPanelV1


class ZwpInputPanelV1Global(Global):
    interface = ZwpInputPanelV1


ZwpInputPanelV1._gen_c()
ZwpInputPanelV1.proxy_class = ZwpInputPanelV1Proxy
ZwpInputPanelV1.resource_class = ZwpInputPanelV1Resource
ZwpInputPanelV1.global_class = ZwpInputPanelV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class ZwpTabletV2(Interface):
    """Graphics tablet device

    The wp_tablet interface represents one graphics tablet device. The tablet
    interface itself does not generate events; all events are generated by
    wp_tablet_tool objects when in proximity above a tablet.

    A tablet has a number of static characteristics, e.g. device name and
    pid/vid. These capabilities are sent in an event sequence after the
    wp_tablet_seat.tablet_added event. This initial event sequence is
    terminated by a wp_tablet.done event.
    """

    name = "zwp_tablet_v2"
    version = 1


class ZwpTabletV2Proxy(Proxy):
    interface = ZwpTabletV2

    @ZwpTabletV2.request()
    def destroy(self):
        """Destroy the tablet object

        This destroys the client's resource for this tablet object.
        """
        self._marshal(0)
        self._destroy()


class ZwpTabletV2Resource(Resource):
    interface = ZwpTabletV2

    @ZwpTabletV2.event(
        Argument(ArgumentType.String),
    )
    def name(self, name):
        """Tablet device name

        This event is sent in the initial burst of events before the
        wp_tablet.done event.

        :param name:
            the device name
        :type name:
            `ArgumentType.String`
        """
        self._post_event(0, name)

    @ZwpTabletV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def id(self, vid, pid):
        """Tablet device usb vendor/product id

        This event is sent in the initial burst of events before the
        wp_tablet.done event.

        :param vid:
            USB vendor id
        :type vid:
            `ArgumentType.Uint`
        :param pid:
            USB product id
        :type pid:
            `ArgumentType.Uint`
        """
        self._post_event(1, vid, pid)

    @ZwpTabletV2.event(
        Argument(ArgumentType.String),
    )
    def path(self, path):
        """Path to the device

        A system-specific device path that indicates which device is behind
        this wp_tablet. This information may be used to gather additional
        information about the device, e.g. through libwacom.

        A device may have more than one device path. If so, multiple
        wp_tablet.path events are sent. A device may be emulated and not have a
        device path, and in that case this event will not be sent.

        The format of the path is unspecified, it may be a device node, a sysfs
        path, or some other identifier. It is up to the client to identify the
        string provided.

        This event is sent in the initial burst of events before the
        wp_tablet.done event.

        :param path:
            path to local device
        :type path:
            `ArgumentType.String`
        """
        self._post_event(2, path)

    @ZwpTabletV2.event()
    def done(self):
        """Tablet description events sequence complete

        This event is sent immediately to signal the end of the initial burst
        of descriptive events. A client may consider the static description of
        the tablet to be complete and finalize initialization of the tablet.
        """
        self._post_event(3)

    @ZwpTabletV2.event()
    def removed(self):
        """Tablet removed event

        Sent when the tablet has been removed from the system. When a tablet is
        removed, some tools may be removed.

        When this event is received, the client must wp_tablet.destroy the
        object.
        """
        self._post_event(4)


class ZwpTabletV2Global(Global):
    interface = ZwpTabletV2


ZwpTabletV2._gen_c()
ZwpTabletV2.proxy_class = ZwpTabletV2Proxy
ZwpTabletV2.resource_class = ZwpTabletV2Resource
ZwpTabletV2.global_class = ZwpTabletV2Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2012, 2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSeat
from ..wayland import WlSurface


class ZwpTextInputV1(Interface):
    """Text input

    An object used for text input. Adds support for text input and input
    methods to applications. A text_input object is created from a
    wl_text_input_manager and corresponds typically to a text entry in an
    application.

    Requests are used to activate/deactivate the text_input object and set
    state information like surrounding and selected text or the content type.
    The information about entered text is sent to the text_input object via the
    pre-edit and commit events. Using this interface removes the need for
    applications to directly process hardware key events and compose text out
    of them.

    Text is generally UTF-8 encoded, indices and lengths are in bytes.

    Serials are used to synchronize the state between the text input and an
    input method. New serials are sent by the text input in the commit_state
    request and are used by the input method to indicate the known text input
    state in events like preedit_string, commit_string, and keysym. The text
    input can then ignore events from the input method which are based on an
    outdated state (for example after a reset).

    Warning! The protocol described in this file is experimental and backward
    incompatible changes may be made. Backward compatible changes may be added
    together with the corresponding interface version bump. Backward
    incompatible changes are done by bumping the version number in the protocol
    and interface names and resetting the interface version. Once the protocol
    is to be declared stable, the 'z' prefix and the version number in the
    protocol and interface names are removed and the interface version number
    is reset.
    """

    name = "zwp_text_input_v1"
    version = 1

    class content_hint(enum.IntFlag):
        none = 0x0
        default = 0x7
        password = 0xC0
        auto_completion = 0x1
        auto_correction = 0x2
        auto_capitalization = 0x4
        lowercase = 0x8
        uppercase = 0x10
        titlecase = 0x20
        hidden_text = 0x40
        sensitive_data = 0x80
        latin = 0x100
        multiline = 0x200

    class content_purpose(enum.IntEnum):
        normal = 0
        alpha = 1
        digits = 2
        number = 3
        phone = 4
        url = 5
        email = 6
        name_ = 7
        password = 8
        date = 9
        time = 10
        datetime = 11
        terminal = 12

    class preedit_style(enum.IntEnum):
        default = 0
        none = 1
        active = 2
        inactive = 3
        highlight = 4
        underline = 5
        selection = 6
        incorrect = 7

    class text_direction(enum.IntEnum):
        auto = 0
        ltr = 1
        rtl = 2


class ZwpTextInputV1Proxy(Proxy):
    interface = ZwpTextInputV1

    @ZwpTextInputV1.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def activate(self, seat, surface):
        """Request activation

        Requests the text_input object to be activated (typically when the text
        entry gets focus).

        The seat argument is a :class:`~pywayland.protocol.wayland.WlSeat`
        which maintains the focus for this activation. The surface argument is
        a :class:`~pywayland.protocol.wayland.WlSurface` assigned to the
        text_input object and tracked for focus lost. The enter event is
        emitted on successful activation.

        :param seat:
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._marshal(0, seat, surface)

    @ZwpTextInputV1.request(
        Argument(ArgumentType.Object, interface=WlSeat),
    )
    def deactivate(self, seat):
        """Request deactivation

        Requests the text_input object to be deactivated (typically when the
        text entry lost focus). The seat argument is a
        :class:`~pywayland.protocol.wayland.WlSeat` which was used for
        activation.

        :param seat:
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        """
        self._marshal(1, seat)

    @ZwpTextInputV1.request()
    def show_input_panel(self):
        """Show input panels

        Requests input panels (virtual keyboard) to show.
        """
        self._marshal(2)

    @ZwpTextInputV1.request()
    def hide_input_panel(self):
        """Hide input panels

        Requests input panels (virtual keyboard) to hide.
        """
        self._marshal(3)

    @ZwpTextInputV1.request()
    def reset(self):
        """Reset

        Should be called by an editor widget when the input state should be
        reset, for example after the text was changed outside of the normal
        input method flow.
        """
        self._marshal(4)

    @ZwpTextInputV1.request(
        Argument(ArgumentType.String),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def set_surrounding_text(self, text, cursor, anchor):
        """Sets the surrounding text

        Sets the plain surrounding text around the input position. Text is
        UTF-8 encoded. Cursor is the byte offset within the surrounding text.
        Anchor is the byte offset of the selection anchor within the
        surrounding text. If there is no selected text anchor, then it is the
        same as cursor.

        :param text:
        :type text:
            `ArgumentType.String`
        :param cursor:
        :type cursor:
            `ArgumentType.Uint`
        :param anchor:
        :type anchor:
            `ArgumentType.Uint`
        """
        self._marshal(5, text, cursor, anchor)

    @ZwpTextInputV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def set_content_type(self, hint, purpose):
        """Set content purpose and hint

        Sets the content purpose and content hint. While the purpose is the
        basic purpose of an input field, the hint flags allow to modify some of
        the behavior.

        When no content type is explicitly set, a normal content purpose with
        default hints (auto completion, auto correction, auto capitalization)
        should be assumed.

        :param hint:
        :type hint:
            `ArgumentType.Uint`
        :param purpose:
        :type purpose:
            `ArgumentType.Uint`
        """
        self._marshal(6, hint, purpose)

    @ZwpTextInputV1.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_cursor_rectangle(self, x, y, width, height):
        """set_cursor_rectangle

        :param x:
        :type x:
            `ArgumentType.Int`
        :param y:
        :type y:
            `ArgumentType.Int`
        :param width:
        :type width:
            `ArgumentType.Int`
        :param height:
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(7, x, y, width, height)

    @ZwpTextInputV1.request(
        Argument(ArgumentType.String),
    )
    def set_preferred_language(self, language):
        """Sets preferred language

        Sets a specific language. This allows for example a virtual keyboard to
        show a language specific layout. The "language" argument is an RFC-3066
        format language tag.

        It could be used for example in a word processor to indicate the
        language of the currently edited document or in an instant message
        application which tracks languages of contacts.

        :param language:
        :type language:
            `ArgumentType.String`
        """
        self._marshal(8, language)

    @ZwpTextInputV1.request(
        Argument(ArgumentType.Uint),
    )
    def commit_state(self, serial):
        """commit_state

        :param serial:
            used to identify the known state
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(9, serial)

    @ZwpTextInputV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def invoke_action(self, button, index):
        """invoke_action

        :param button:
        :type button:
            `ArgumentType.Uint`
        :param index:
        :type index:
            `ArgumentType.Uint`
        """
        self._marshal(10, button, index)


class ZwpTextInputV1Resource(Resource):
    interface = ZwpTextInputV1

    @ZwpTextInputV1.event(
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def enter(self, surface):
        """Enter event

        Notify the text_input object when it received focus. Typically in
        response to an activate request.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._post_event(0, surface)

    @ZwpTextInputV1.event()
    def leave(self):
        """Leave event

        Notify the text_input object when it lost focus. Either in response to
        a deactivate request or when the assigned surface lost focus or was
        destroyed.
        """
        self._post_event(1)

    @ZwpTextInputV1.event(
        Argument(ArgumentType.Array),
    )
    def modifiers_map(self, map):
        """Modifiers map

        Transfer an array of 0-terminated modifier names. The position in the
        array is the index of the modifier as used in the modifiers bitmask in
        the keysym event.

        :param map:
        :type map:
            `ArgumentType.Array`
        """
        self._post_event(2, map)

    @ZwpTextInputV1.event(
        Argument(ArgumentType.Uint),
    )
    def input_panel_state(self, state):
        """State of the input panel

        Notify when the visibility state of the input panel changed.

        :param state:
        :type state:
            `ArgumentType.Uint`
        """
        self._post_event(3, state)

    @ZwpTextInputV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String),
        Argument(ArgumentType.String),
    )
    def preedit_string(self, serial, text, commit):
        """Pre-edit

        Notify when a new composing text (pre-edit) should be set around the
        current cursor position. Any previously set composing text should be
        removed.

        The commit text can be used to replace the preedit text on reset (for
        example on unfocus).

        The text input should also handle all preedit_style and preedit_cursor
        events occurring directly before preedit_string.

        :param serial:
            serial of the latest known text input state
        :type serial:
            `ArgumentType.Uint`
        :param text:
        :type text:
            `ArgumentType.String`
        :param commit:
        :type commit:
            `ArgumentType.String`
        """
        self._post_event(4, serial, text, commit)

    @ZwpTextInputV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def preedit_styling(self, index, length, style):
        """Pre-edit styling

        Sets styling information on composing text. The style is applied for
        length bytes from index relative to the beginning of the composing text
        (as byte offset). Multiple styles can be applied to a composing text by
        sending multiple preedit_styling events.

        This event is handled as part of a following preedit_string event.

        :param index:
        :type index:
            `ArgumentType.Uint`
        :param length:
        :type length:
            `ArgumentType.Uint`
        :param style:
        :type style:
            `ArgumentType.Uint`
        """
        self._post_event(5, index, length, style)

    @ZwpTextInputV1.event(
        Argument(ArgumentType.Int),
    )
    def preedit_cursor(self, index):
        """Pre-edit cursor

        Sets the cursor position inside the composing text (as byte offset)
        relative to the start of the composing text. When index is a negative
        number no cursor is shown.

        This event is handled as part of a following preedit_string event.

        :param index:
        :type index:
            `ArgumentType.Int`
        """
        self._post_event(6, index)

    @ZwpTextInputV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String),
    )
    def commit_string(self, serial, text):
        """Commit

        Notify when text should be inserted into the editor widget. The text to
        commit could be either just a single character after a key press or the
        result of some composing (pre-edit). It could also be an empty text
        when some text should be removed (see delete_surrounding_text) or when
        the input cursor should be moved (see cursor_position).

        Any previously set composing text should be removed.

        :param serial:
            serial of the latest known text input state
        :type serial:
            `ArgumentType.Uint`
        :param text:
        :type text:
            `ArgumentType.String`
        """
        self._post_event(7, serial, text)

    @ZwpTextInputV1.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def cursor_position(self, index, anchor):
        """Set cursor to new position

        Notify when the cursor or anchor position should be modified.

        This event should be handled as part of a following commit_string
        event.

        :param index:
        :type index:
            `ArgumentType.Int`
        :param anchor:
        :type anchor:
            `ArgumentType.Int`
        """
        self._post_event(8, index, anchor)

    @ZwpTextInputV1.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
    )
    def delete_surrounding_text(self, index, length):
        """Delete surrounding text

        Notify when the text around the current cursor position should be
        deleted.

        Index is relative to the current cursor (in bytes). Length is the
        length of deleted text (in bytes).

        This event should be handled as part of a following commit_string
        event.

        :param index:
        :type index:
            `ArgumentType.Int`
        :param length:
        :type length:
            `ArgumentType.Uint`
        """
        self._post_event(9, index, length)

    @ZwpTextInputV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def keysym(self, serial, time, sym, state, modifiers):
        """Keysym

        Notify when a key event was sent. Key events should not be used for
        normal text input operations, which should be done with commit_string,
        delete_surrounding_text, etc. The key event follows the
        :class:`~pywayland.protocol.wayland.WlKeyboard` key event convention.
        Sym is an XKB keysym, state a
        :class:`~pywayland.protocol.wayland.WlKeyboard` key_state. Modifiers
        are a mask for effective modifiers (where the modifier indices are set
        by the modifiers_map event)

        :param serial:
            serial of the latest known text input state
        :type serial:
            `ArgumentType.Uint`
        :param time:
        :type time:
            `ArgumentType.Uint`
        :param sym:
        :type sym:
            `ArgumentType.Uint`
        :param state:
        :type state:
            `ArgumentType.Uint`
        :param modifiers:
        :type modifiers:
            `ArgumentType.Uint`
        """
        self._post_event(10, serial, time, sym, state, modifiers)

    @ZwpTextInputV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String),
    )
    def language(self, serial, language):
        """Language

        Sets the language of the input text. The "language" argument is an
        RFC-3066 format language tag.

        :param serial:
            serial of the latest known text input state
        :type serial:
            `ArgumentType.Uint`
        :param language:
        :type language:
            `ArgumentType.String`
        """
        self._post_event(11, serial, language)

    @ZwpTextInputV1.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def text_direction(self, serial, direction):
        """Text direction

        Sets the text direction of input text.

        It is mainly needed for showing an input cursor on the correct side of
        the editor when there is no input done yet and making sure neutral
        direction text is laid out properly.

        :param serial:
            serial of the latest known text input state
        :type serial:
            `ArgumentType.Uint`
        :param direction:
        :type direction:
            `ArgumentType.Uint`
        """
        self._post_event(12, serial, direction)


class ZwpTextInputV1Global(Global):
    interface = ZwpTextInputV1


ZwpTextInputV1._gen_c()
ZwpTextInputV1.proxy_class = ZwpTextInputV1Proxy
ZwpTextInputV1.resource_class = ZwpTextInputV1Resource
ZwpTextInputV1.global_class = ZwpTextInputV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools
from weakref import WeakKeyDictionary
from typing import TYPE_CHECKING, Optional

from pywayland import ffi, lib

if TYPE_CHECKING:
    from pywayland.client import Display  # noqa: F401

weakkeydict: WeakKeyDictionary = WeakKeyDictionary()


def _event_queue_destroy(display: "Display", cdata: "ffi.QueueCData") -> None:
    # we should be careful that the display is still around
    if display._ptr is None:
        return
    lib.wl_event_queue_destroy(cdata)


class EventQueue:
    """A queue for wl_proxy object events.

    Event queues allows the events on a display to be handled in a thread-safe
    manner. See :class:`~pywayland.client.Display` for details.

    :param display:
        The display object that the event queue is connected to.
    :type display:
        :class:`~pywayland.client.Display`
    """

    def __init__(self, display: "Display") -> None:
        """Constructor for the EventQueue object"""
        # check that we attach to a valid display
        if display._ptr is None or display._ptr == ffi.NULL:
            raise ValueError("Display object not connected")

        ptr = lib.wl_display_create_queue(display._ptr)

        # create a destructor, save data and display
        destructor = functools.partial(_event_queue_destroy, display)
        self._ptr: Optional["ffi.QueueCData"] = ffi.gc(ptr, destructor)
        self._display: Optional["Display"] = display

        display._children.add(self)

        weakkeydict[self._ptr] = display

    @property
    def destroyed(self) -> bool:
        """Determine the state of the event queue"""
        return self._ptr is None

    def destroy(self) -> None:
        """Destroy an event queue

        Destroy the given event queue. Any pending event on that queue is
        discarded.

        The wl_display object used to create the queue should not be destroyed
        until all event queues created with it are destroyed with this
        function.
        """
        if self._ptr is not None and self._display is not None:
            ffi.release(self._ptr)

            # delete the pointer and the reference to the display
            self._ptr = None
            self._display = None



# This file has been autogenerated by the pywayland scanner

# Copyright © 2015, 2016 Red Hat
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSeat
from .zwp_primary_selection_device_v1 import ZwpPrimarySelectionDeviceV1
from .zwp_primary_selection_source_v1 import ZwpPrimarySelectionSourceV1


class ZwpPrimarySelectionDeviceManagerV1(Interface):
    """X primary selection emulation

    The primary selection device manager is a singleton global object that
    provides access to the primary selection. It allows to create
    wp_primary_selection_source objects, as well as retrieving the per-seat
    wp_primary_selection_device objects.
    """

    name = "zwp_primary_selection_device_manager_v1"
    version = 1


class ZwpPrimarySelectionDeviceManagerV1Proxy(Proxy):
    interface = ZwpPrimarySelectionDeviceManagerV1

    @ZwpPrimarySelectionDeviceManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZwpPrimarySelectionSourceV1),
    )
    def create_source(self):
        """Create a new primary selection source

        Create a new primary selection source.

        :returns:
            :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionSourceV1`
        """
        id = self._marshal_constructor(0, ZwpPrimarySelectionSourceV1)
        return id

    @ZwpPrimarySelectionDeviceManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZwpPrimarySelectionDeviceV1),
        Argument(ArgumentType.Object, interface=WlSeat),
    )
    def get_device(self, seat):
        """Create a new primary selection device

        Create a new data device for a given seat.

        :param seat:
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :returns:
            :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionDeviceV1`
        """
        id = self._marshal_constructor(1, ZwpPrimarySelectionDeviceV1, seat)
        return id

    @ZwpPrimarySelectionDeviceManagerV1.request()
    def destroy(self):
        """Destroy the primary selection device manager

        Destroy the primary selection device manager.
        """
        self._marshal(2)
        self._destroy()


class ZwpPrimarySelectionDeviceManagerV1Resource(Resource):
    interface = ZwpPrimarySelectionDeviceManagerV1


class ZwpPrimarySelectionDeviceManagerV1Global(Global):
    interface = ZwpPrimarySelectionDeviceManagerV1


ZwpPrimarySelectionDeviceManagerV1._gen_c()
ZwpPrimarySelectionDeviceManagerV1.proxy_class = ZwpPrimarySelectionDeviceManagerV1Proxy
ZwpPrimarySelectionDeviceManagerV1.resource_class = ZwpPrimarySelectionDeviceManagerV1Resource
ZwpPrimarySelectionDeviceManagerV1.global_class = ZwpPrimarySelectionDeviceManagerV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from typing import Optional
import xml.etree.ElementTree as ET

from .element import Element
from .printer import Printer


@dataclass(frozen=True)
class Description(Element):
    summary: str
    text: Optional[str]

    @classmethod
    def parse(cls, element: ET.Element) -> "Description":
        return cls(
            summary=cls.parse_attribute(element, "summary"),
            text=cls.parse_pcdata(element),
        )

    def output(self, printer: Printer) -> None:
        printer.doc('"""{}'.format(self.summary.capitalize()))
        if self.text:
            printer()
            printer.docstring(self.text)



# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .zwp_tablet_pad_ring_v2 import ZwpTabletPadRingV2
from .zwp_tablet_pad_strip_v2 import ZwpTabletPadStripV2


class ZwpTabletPadGroupV2(Interface):
    """A set of buttons, rings and strips

    A pad group describes a distinct (sub)set of buttons, rings and strips
    present in the tablet. The criteria of this grouping is usually positional,
    eg. if a tablet has buttons on the left and right side, 2 groups will be
    presented. The physical arrangement of groups is undisclosed and may change
    on the fly.

    Pad groups will announce their features during pad initialization. Between
    the corresponding wp_tablet_pad.group event and wp_tablet_pad_group.done,
    the pad group will announce the buttons, rings and strips contained in it,
    plus the number of supported modes.

    Modes are a mechanism to allow multiple groups of actions for every element
    in the pad group. The number of groups and available modes in each is
    persistent across device plugs. The current mode is user-switchable, it
    will be announced through the wp_tablet_pad_group.mode_switch event both
    whenever it is switched, and after wp_tablet_pad.enter.

    The current mode logically applies to all elements in the pad group,
    although it is at clients' discretion whether to actually perform different
    actions, and/or issue the respective .set_feedback requests to notify the
    compositor. See the wp_tablet_pad_group.mode_switch event for more details.
    """

    name = "zwp_tablet_pad_group_v2"
    version = 1


class ZwpTabletPadGroupV2Proxy(Proxy):
    interface = ZwpTabletPadGroupV2

    @ZwpTabletPadGroupV2.request()
    def destroy(self):
        """Destroy the pad object

        Destroy the wp_tablet_pad_group object. Objects created from this
        object are unaffected and should be destroyed separately.
        """
        self._marshal(0)
        self._destroy()


class ZwpTabletPadGroupV2Resource(Resource):
    interface = ZwpTabletPadGroupV2

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.Array),
    )
    def buttons(self, buttons):
        """Buttons announced

        Sent on wp_tablet_pad_group initialization to announce the available
        buttons in the group. Button indices start at 0, a button may only be
        in one group at a time.

        This event is first sent in the initial burst of events before the
        wp_tablet_pad_group.done event.

        Some buttons are reserved by the compositor. These buttons may not be
        assigned to any wp_tablet_pad_group. Compositors may broadcast this
        event in the case of changes to the mapping of these reserved buttons.
        If the compositor happens to reserve all buttons in a group, this event
        will be sent with an empty array.

        :param buttons:
            buttons in this group
        :type buttons:
            `ArgumentType.Array`
        """
        self._post_event(0, buttons)

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletPadRingV2),
    )
    def ring(self, ring):
        """Ring announced

        Sent on wp_tablet_pad_group initialization to announce available rings.
        One event is sent for each ring available on this pad group.

        This event is sent in the initial burst of events before the
        wp_tablet_pad_group.done event.

        :param ring:
        :type ring:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletPadRingV2`
        """
        self._post_event(1, ring)

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletPadStripV2),
    )
    def strip(self, strip):
        """Strip announced

        Sent on wp_tablet_pad initialization to announce available strips. One
        event is sent for each strip available on this pad group.

        This event is sent in the initial burst of events before the
        wp_tablet_pad_group.done event.

        :param strip:
        :type strip:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletPadStripV2`
        """
        self._post_event(2, strip)

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.Uint),
    )
    def modes(self, modes):
        """Mode-switch ability announced

        Sent on wp_tablet_pad_group initialization to announce that the pad
        group may switch between modes. A client may use a mode to store a
        specific configuration for buttons, rings and strips and use the
        wl_tablet_pad_group.mode_switch event to toggle between these
        configurations. Mode indices start at 0.

        Switching modes is compositor-dependent. See the
        wp_tablet_pad_group.mode_switch event for more details.

        This event is sent in the initial burst of events before the
        wp_tablet_pad_group.done event. This event is only sent when more than
        more than one mode is available.

        :param modes:
            the number of modes
        :type modes:
            `ArgumentType.Uint`
        """
        self._post_event(3, modes)

    @ZwpTabletPadGroupV2.event()
    def done(self):
        """Tablet group description events sequence complete

        This event is sent immediately to signal the end of the initial burst
        of descriptive events. A client may consider the static description of
        the tablet to be complete and finalize initialization of the tablet
        group.
        """
        self._post_event(4)

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def mode_switch(self, time, serial, mode):
        """Mode switch event

        Notification that the mode was switched.

        A mode applies to all buttons, rings and strips in a group
        simultaneously, but a client is not required to assign different
        actions for each mode. For example, a client may have mode-specific
        button mappings but map the ring to vertical scrolling in all modes.
        Mode indices start at 0.

        Switching modes is compositor-dependent. The compositor may provide
        visual cues to the client about the mode, e.g. by toggling LEDs on the
        tablet device. Mode-switching may be software-controlled or controlled
        by one or more physical buttons. For example, on a Wacom Intuos Pro,
        the button inside the ring may be assigned to switch between modes.

        The compositor will also send this event after wp_tablet_pad.enter on
        each group in order to notify of the current mode. Groups that only
        feature one mode will use mode=0 when emitting this event.

        If a button action in the new mode differs from the action in the
        previous mode, the client should immediately issue a
        wp_tablet_pad.set_feedback request for each changed button.

        If a ring or strip action in the new mode differs from the action in
        the previous mode, the client should immediately issue a
        wp_tablet_ring.set_feedback or wp_tablet_strip.set_feedback request for
        each changed ring or strip.

        :param time:
            the time of the event with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param mode:
            the new mode of the pad
        :type mode:
            `ArgumentType.Uint`
        """
        self._post_event(5, time, serial, mode)


class ZwpTabletPadGroupV2Global(Global):
    interface = ZwpTabletPadGroupV2


ZwpTabletPadGroupV2._gen_c()
ZwpTabletPadGroupV2.proxy_class = ZwpTabletPadGroupV2Proxy
ZwpTabletPadGroupV2.resource_class = ZwpTabletPadGroupV2Resource
ZwpTabletPadGroupV2.global_class = ZwpTabletPadGroupV2Global



# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_requests import WlRequests


class WlCore(Interface):
    """Interface object

    The interface object with the most basic content.
    """

    name = "wl_core"
    version = 1

    class the_enum(enum.IntEnum):
        zero = 0
        one = 1
        hex_two = 0x2


class WlCoreProxy(Proxy):
    interface = WlCore

    @WlCore.request(
        Argument(ArgumentType.NewId, interface=WlCore),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Fixed),
    )
    def make_request(self, the_int, the_uint, the_fixed):
        """A request

        The request asks the server for an event.

        :param the_int:
            the arg summary
        :type the_int:
            `ArgumentType.Int`
        :param the_uint:
        :type the_uint:
            `ArgumentType.Uint`
        :param the_fixed:
        :type the_fixed:
            `ArgumentType.Fixed`
        :returns:
            :class:`WlCore`
        """
        id = self._marshal_constructor(0, WlCore, the_int, the_uint, the_fixed)
        return id

    @WlCore.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.NewId, interface=WlCore),
    )
    def make_request2(self, the_int, the_uint, the_fixed):
        """A request

        The request asks the server for an event but move the args around.

        :param the_int:
            the arg summary
        :type the_int:
            `ArgumentType.Int`
        :param the_uint:
        :type the_uint:
            `ArgumentType.Uint`
        :param the_fixed:
        :type the_fixed:
            `ArgumentType.Fixed`
        :returns:
            :class:`WlCore` -- a :class:`WlCore` object
        """
        id = self._marshal_constructor(1, WlCore, the_int, the_uint, the_fixed)
        return id


class WlCoreResource(Resource):
    interface = WlCore

    @WlCore.event(
        Argument(ArgumentType.NewId, interface=WlCore),
        Argument(ArgumentType.Object, interface=WlRequests),
    )
    def send_event(self, id, object):
        """A :class:`WlCore` event

        Send an event, but also put in some docs for our interface
        :class:`WlCore`, some other interface wl_other, a local function call
        :func:`WlCore.func()`, and another function call wl_other.func.

        :param id:
            a :class:`WlCore` object
        :type id:
            :class:`WlCore`
        :param object:
            a :class:`~pywayland.protocol.scanner_test.WlRequests` object
        :type object:
            :class:`~pywayland.protocol.scanner_test.WlRequests`
        """
        self._post_event(0, id, object)


class WlCoreGlobal(Global):
    interface = WlCore


WlCore._gen_c()
WlCore.proxy_class = WlCoreProxy
WlCore.resource_class = WlCoreResource
WlCore.global_class = WlCoreGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import itertools
import xml.etree.ElementTree as ET
from dataclasses import dataclass
from typing import Dict, List, Optional

from .description import Description
from .element import Element
from .enum import Enum
from .event import Event
from .printer import Printer
from .request import Request


@dataclass(frozen=True)
class Interface(Element):
    name: str
    version: str
    description: Optional[Description]
    enum: List[Enum]
    event: List[Event]
    request: List[Request]

    @classmethod
    def parse(cls, element: ET.Element) -> "Interface":
        """Scanner for interface objects

        Required attributes: `name` and `version`

        Child elements: `description`, `request`, `event`, `enum`
        """
        name = cls.parse_attribute(element, "name")
        return cls(
            name=name,
            version=cls.parse_attribute(element, "version"),
            description=cls.parse_optional_child(element, Description, "description"),
            enum=cls.parse_repeated_child(element, Enum, "enum"),
            event=cls.parse_repeated_child(element, Event, "event"),
            request=cls.parse_repeated_child(element, Request, "request"),
        )

    @property
    def class_name(self) -> str:
        """Returns the name of the class of the interface

        Camel cases the name of the interface, to be used as the class name.
        """
        return "".join(x.capitalize() for x in self.name.split("_"))

    def output(self, printer: Printer, module_imports: Dict[str, str]) -> None:
        """Generate the output for the interface to the printer"""
        # Imports
        imports = set(
            _import
            for method in itertools.chain(self.request, self.event)
            for _import in method.imports(self.name, module_imports)
        )

        needs_argument_type = any(
            len(method.arg) > 0 for method in self.request
        ) or any(len(method.arg) > 0 for method in self.event)

        if self.enum:
            printer("import enum")
            printer()
        if needs_argument_type:
            printer(
                "from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource"
            )
        else:
            printer(
                "from pywayland.protocol_core import Global, Interface, Proxy, Resource"
            )

        for module, import_ in sorted(imports):
            printer("from {} import {}".format(module, import_))
        printer()
        printer()

        # Class definition
        printer("class {}(Interface):".format(self.class_name))
        with printer.indented():
            # Docstring
            if self.description:
                self.description.output(printer)
                printer('"""')
                printer()

            # Class attributes
            printer('name = "{}"'.format(self.name))
            printer("version = {}".format(self.version))

            # Enums
            for enum in self.enum:
                printer()
                enum.output(printer)

        proxy_class_name = f"{self.class_name}Proxy"
        resource_class_name = f"{self.class_name}Resource"
        global_class_name = f"{self.class_name}Global"

        printer()
        printer()
        printer(f"class {proxy_class_name}(Proxy):")
        with printer.indented():
            printer(f"interface = {self.class_name}")
            for opcode, request in enumerate(self.request):
                printer()
                request.output(printer, opcode, self.class_name, module_imports)

        printer()
        printer()
        printer(f"class {resource_class_name}(Resource):")
        with printer.indented():
            printer(f"interface = {self.class_name}")
            for opcode, event in enumerate(self.event):
                printer()
                event.output(printer, opcode, self.class_name, module_imports)

        printer()
        printer()
        printer(f"class {global_class_name}(Global):")
        with printer.indented():
            printer(f"interface = {self.class_name}")

        printer()
        printer()
        printer("{}._gen_c()".format(self.class_name))
        printer(f"{self.class_name}.proxy_class = {proxy_class_name}")
        printer(f"{self.class_name}.resource_class = {resource_class_name}")
        printer(f"{self.class_name}.global_class = {global_class_name}")



# This file has been autogenerated by the pywayland scanner

# Copyright © 2015 Samsung Electronics Co., Ltd
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Global, Interface, Proxy, Resource


class ZwpIdleInhibitorV1(Interface):
    """Context object for inhibiting idle behavior

    An idle inhibitor prevents the output that the associated surface is
    visible on from being set to a state where it is not visually usable due to
    lack of user interaction (e.g. blanked, dimmed, locked, set to power save,
    etc.)  Any screensaver processes are also blocked from displaying.

    If the surface is destroyed, unmapped, becomes occluded, loses visibility,
    or otherwise becomes not visually relevant for the user, the idle inhibitor
    will not be honored by the compositor; if the surface subsequently regains
    visibility the inhibitor takes effect once again. Likewise, the inhibitor
    isn't honored if the system was already idled at the time the inhibitor was
    established, although if the system later de-idles and re-idles the
    inhibitor will take effect.
    """

    name = "zwp_idle_inhibitor_v1"
    version = 1


class ZwpIdleInhibitorV1Proxy(Proxy):
    interface = ZwpIdleInhibitorV1

    @ZwpIdleInhibitorV1.request()
    def destroy(self):
        """Destroy the idle inhibitor object

        Remove the inhibitor effect from the associated
        :class:`~pywayland.protocol.wayland.WlSurface`.
        """
        self._marshal(0)
        self._destroy()


class ZwpIdleInhibitorV1Resource(Resource):
    interface = ZwpIdleInhibitorV1


class ZwpIdleInhibitorV1Global(Global):
    interface = ZwpIdleInhibitorV1


ZwpIdleInhibitorV1._gen_c()
ZwpIdleInhibitorV1.proxy_class = ZwpIdleInhibitorV1Proxy
ZwpIdleInhibitorV1.resource_class = ZwpIdleInhibitorV1Resource
ZwpIdleInhibitorV1.global_class = ZwpIdleInhibitorV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2018 NXP
# Copyright © 2019 Status Research & Development GmbH.
# Copyright © 2021 Xaver Hugl
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class WpDrmLeaseV1(Interface):
    """A drm lease

    A DRM lease object is used to transfer the DRM file descriptor to the
    client and manage the lifetime of the lease.

    Some time after the :class:`WpDrmLeaseV1` object is created, the compositor
    will reply with the lease request's result. If the lease request is
    granted, the compositor will send a lease_fd event. If the lease request is
    denied, the compositor will send a finished event without a lease_fd event.
    """

    name = "wp_drm_lease_v1"
    version = 1


class WpDrmLeaseV1Proxy(Proxy):
    interface = WpDrmLeaseV1

    @WpDrmLeaseV1.request()
    def destroy(self):
        """Destroys the lease object

        The client should send this to indicate that it no longer wishes to use
        this lease. The compositor should use drmModeRevokeLease on the
        appropriate file descriptor, if necessary.
        """
        self._marshal(0)
        self._destroy()


class WpDrmLeaseV1Resource(Resource):
    interface = WpDrmLeaseV1

    @WpDrmLeaseV1.event(
        Argument(ArgumentType.FileDescriptor),
    )
    def lease_fd(self, leased_fd):
        """Shares the drm file descriptor

        This event returns a file descriptor suitable for use with DRM-related
        ioctls. The client should use drmModeGetLease to enumerate the DRM
        objects which have been leased to them. The compositor guarantees it
        will not use the leased DRM objects itself until it sends the finished
        event. If the compositor cannot or will not grant a lease for the
        requested connectors, it will not send this event, instead sending the
        finished event.

        The compositor will send this event at most once during this objects
        lifetime.

        :param leased_fd:
            leased DRM file descriptor
        :type leased_fd:
            `ArgumentType.FileDescriptor`
        """
        self._post_event(0, leased_fd)

    @WpDrmLeaseV1.event()
    def finished(self):
        """Sent when the lease has been revoked

        The compositor uses this event to either reject a lease request, or if
        it previously sent a lease_fd, to notify the client that the lease has
        been revoked. If the client requires a new lease, they should destroy
        this object and submit a new lease request. The compositor will send no
        further events for this object after sending the finish event.
        Compositors should revoke the lease when any of the leased resources
        become unavailable, namely when a hot-unplug occurs or when the
        compositor loses DRM master.
        """
        self._post_event(1)


class WpDrmLeaseV1Global(Global):
    interface = WpDrmLeaseV1


WpDrmLeaseV1._gen_c()
WpDrmLeaseV1.proxy_class = WpDrmLeaseV1Proxy
WpDrmLeaseV1.resource_class = WpDrmLeaseV1Resource
WpDrmLeaseV1.global_class = WpDrmLeaseV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_callback import WlCallback
from .wl_registry import WlRegistry


class WlDisplay(Interface):
    """Core global object

    The core global object.  This is a special singleton object.  It is used
    for internal Wayland protocol features.
    """

    name = "wl_display"
    version = 1

    class error(enum.IntEnum):
        invalid_object = 0
        invalid_method = 1
        no_memory = 2
        implementation = 3


class WlDisplayProxy(Proxy):
    interface = WlDisplay

    @WlDisplay.request(
        Argument(ArgumentType.NewId, interface=WlCallback),
    )
    def sync(self):
        """Asynchronous roundtrip

        The sync request asks the server to emit the 'done' event on the
        returned :class:`~pywayland.protocol.wayland.WlCallback` object.  Since
        requests are handled in-order and events are delivered in-order, this
        can be used as a barrier to ensure all previous requests and the
        resulting events have been handled.

        The object returned by this request will be destroyed by the compositor
        after the callback is fired and as such the client must not attempt to
        use it after that point.

        The callback_data passed in the callback is the event serial.

        :returns:
            :class:`~pywayland.protocol.wayland.WlCallback` -- callback object
            for the sync request
        """
        callback = self._marshal_constructor(0, WlCallback)
        return callback

    @WlDisplay.request(
        Argument(ArgumentType.NewId, interface=WlRegistry),
    )
    def get_registry(self):
        """Get global registry object

        This request creates a registry object that allows the client to list
        and bind the global objects available from the compositor.

        It should be noted that the server side resources consumed in response
        to a get_registry request can only be released when the client
        disconnects, not when the client side proxy is destroyed. Therefore,
        clients should invoke get_registry as infrequently as possible to avoid
        wasting memory.

        :returns:
            :class:`~pywayland.protocol.wayland.WlRegistry` -- global registry
            object
        """
        registry = self._marshal_constructor(1, WlRegistry)
        return registry


class WlDisplayResource(Resource):
    interface = WlDisplay

    @WlDisplay.event(
        Argument(ArgumentType.Object),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String),
    )
    def error(self, object_id, code, message):
        """Fatal error event

        The error event is sent out when a fatal (non-recoverable) error has
        occurred.  The object_id argument is the object where the error
        occurred, most often in response to a request to that object.  The code
        identifies the error and is defined by the object interface.  As such,
        each interface defines its own set of error codes.  The message is a
        brief description of the error, for (debugging) convenience.

        :param object_id:
            object where the error occurred
        :type object_id:
            `ArgumentType.Object`
        :param code:
            error code
        :type code:
            `ArgumentType.Uint`
        :param message:
            error description
        :type message:
            `ArgumentType.String`
        """
        self._post_event(0, object_id, code, message)

    @WlDisplay.event(
        Argument(ArgumentType.Uint),
    )
    def delete_id(self, id):
        """Acknowledge object id deletion

        This event is used internally by the object ID management logic. When a
        client deletes an object that it had created, the server will send this
        event to acknowledge that it has seen the delete request. When the
        client receives this event, it will know that it can safely reuse the
        object ID.

        :param id:
            deleted object ID
        :type id:
            `ArgumentType.Uint`
        """
        self._post_event(1, id)


class WlDisplayGlobal(Global):
    interface = WlDisplay


WlDisplay._gen_c()
WlDisplay.proxy_class = WlDisplayProxy
WlDisplay.resource_class = WlDisplayResource
WlDisplay.global_class = WlDisplayGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import traceback
from typing import Callable, List, Optional, Union

from pywayland.protocol_core.message import Message
from pywayland import ffi, lib

CallbackT = Callable[..., Optional[int]]


# int (*wl_dispatcher_func_t)(const void *, void *, uint32_t, const struct wl_message *, union wl_argument *)
@ffi.def_extern()
def dispatcher_func(data, target, opcode, message, c_args):
    # `data` is the handle to proxy/resource python object
    # `target` is the wl_proxy/wl_resource for self, this should be the same as self._ptr
    # `message` is the wl_message for self._interface.events/requests[opcode]
    # TODO: handle any user_data attached to the wl_proxy/wl_resource

    # get the proxy/resource object from the user data handle
    self = ffi.from_handle(data)

    # get the callback
    func = self.dispatcher[opcode]
    if func is None:
        return 0

    # rebuild the args into python objects
    args = self.dispatcher.messages[opcode].c_to_arguments(c_args)

    try:
        ret = func(self, *args)
    except Exception:
        traceback.print_exc()
        return 0

    if ret is None:
        return 0
    else:
        return ret


# void (*wl_resource_destroy_func_t)(struct wl_resource *resource)
@ffi.def_extern()
def resource_destroy_func(res_ptr):
    # the user data to the resource is the handle to the resource
    resource_handle = lib.wl_resource_get_user_data(res_ptr)
    resource = ffi.from_handle(resource_handle)

    # if the destructor has been set, run it
    func = resource.dispatcher.destructor
    if func is not None:
        func(resource)


class Dispatcher:
    """Dispatches events or requests from an interface

    Handles the dispatching of callbacks from events (for
    :class:`~pywayland.sever.resource.Resource` objects) and requests (for
    :class:`~pywayland.client.proxy.Proxy` objects).  The callbacks for a given
    message can be set and retrieved by indexing the dispatcher with either the
    opcode or the name of the message.

    :param messages:
        List of messages (events or requests)
    :type messages:
        `list`
    :param destructor:
        Create destructor dispatcher (for Resources)
    :type destructor:
        `bool`
    """

    def __init__(self, messages: List[Message], destructor: bool = False) -> None:
        self.messages = messages

        # Create a map of message names to message opcodes
        self._names = {msg.name: opcode for opcode, msg in enumerate(messages)}
        self._callback: List[Optional[CallbackT]] = [None] * len(messages)

        if destructor:
            self.destructor = None

    def __getitem__(self, opcode_or_name: Union[str, int]) -> Optional[CallbackT]:
        if isinstance(opcode_or_name, str):
            opcode_or_name = self._names[opcode_or_name]

        return self._callback[opcode_or_name]

    def __setitem__(self, opcode_or_name: Union[str, int], function: CallbackT) -> None:
        if isinstance(opcode_or_name, str):
            opcode_or_name = self._names[opcode_or_name]
        self._callback[opcode_or_name] = function



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_region import WlRegion
from .wl_surface import WlSurface


class WlCompositor(Interface):
    """The compositor singleton

    A compositor.  This object is a singleton global.  The compositor is in
    charge of combining the contents of multiple surfaces into one displayable
    output.
    """

    name = "wl_compositor"
    version = 4


class WlCompositorProxy(Proxy):
    interface = WlCompositor

    @WlCompositor.request(
        Argument(ArgumentType.NewId, interface=WlSurface),
    )
    def create_surface(self):
        """Create new surface

        Ask the compositor to create a new surface.

        :returns:
            :class:`~pywayland.protocol.wayland.WlSurface` -- the new surface
        """
        id = self._marshal_constructor(0, WlSurface)
        return id

    @WlCompositor.request(
        Argument(ArgumentType.NewId, interface=WlRegion),
    )
    def create_region(self):
        """Create new region

        Ask the compositor to create a new region.

        :returns:
            :class:`~pywayland.protocol.wayland.WlRegion` -- the new region
        """
        id = self._marshal_constructor(1, WlRegion)
        return id


class WlCompositorResource(Resource):
    interface = WlCompositor


class WlCompositorGlobal(Global):
    interface = WlCompositor


WlCompositor._gen_c()
WlCompositor.proxy_class = WlCompositorProxy
WlCompositor.resource_class = WlCompositorResource
WlCompositor.global_class = WlCompositorGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2012, 2013 Intel Corporation
# Copyright © 2015, 2016 Jan Arne Petersen
# Copyright © 2017, 2018 Red Hat, Inc.
# Copyright © 2018       Purism SPC
#
# Permission to use, copy, modify, distribute, and sell this
# software and its documentation for any purpose is hereby granted
# without fee, provided that the above copyright notice appear in
# all copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of
# the copyright holders not be used in advertising or publicity
# pertaining to distribution of the software without specific,
# written prior permission.  The copyright holders make no
# representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied
# warranty.
#
# THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
# SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
# SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
# ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
# THIS SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSeat
from .zwp_text_input_v3 import ZwpTextInputV3


class ZwpTextInputManagerV3(Interface):
    """Text input manager

    A factory for text-input objects. This object is a global singleton.
    """

    name = "zwp_text_input_manager_v3"
    version = 1


class ZwpTextInputManagerV3Proxy(Proxy):
    interface = ZwpTextInputManagerV3

    @ZwpTextInputManagerV3.request()
    def destroy(self):
        """Destroy the wp_text_input_manager

        Destroy the wp_text_input_manager object.
        """
        self._marshal(0)
        self._destroy()

    @ZwpTextInputManagerV3.request(
        Argument(ArgumentType.NewId, interface=ZwpTextInputV3),
        Argument(ArgumentType.Object, interface=WlSeat),
    )
    def get_text_input(self, seat):
        """Create a new text input object

        Creates a new text-input object for a given seat.

        :param seat:
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :returns:
            :class:`~pywayland.protocol.text_input_unstable_v3.ZwpTextInputV3`
        """
        id = self._marshal_constructor(1, ZwpTextInputV3, seat)
        return id


class ZwpTextInputManagerV3Resource(Resource):
    interface = ZwpTextInputManagerV3


class ZwpTextInputManagerV3Global(Global):
    interface = ZwpTextInputManagerV3


ZwpTextInputManagerV3._gen_c()
ZwpTextInputManagerV3.proxy_class = ZwpTextInputManagerV3Proxy
ZwpTextInputManagerV3.resource_class = ZwpTextInputManagerV3Resource
ZwpTextInputManagerV3.global_class = ZwpTextInputManagerV3Global



# Copyright 2015 Sean Vig
# Copyright 2021 Matt Colligan
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from functools import wraps
from typing import Callable, Iterator, Optional

from . import ffi, lib


def ensure_valid(func: Callable) -> Callable:
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        if self._ptr is None:
            raise ValueError(
                "{cls} object has been destroyed".format(cls=self.__class__.__name__)
            )
        return func(self, *args, **kwargs)

    return wrapper


class AnonymousFile:
    """Anonymous file object

    Provides access to anonymous file objects that can be used by Wayland
    clients to render to surfaces.  Uses a method similar to Weston to open an
    anonymous file, so XDG_RUNTIME_DIR must be set for this to work properly.

    This class provides a content manager, that is, it can be used with Python
    ``with`` statements, where the value returned is the file descriptor.
    """

    def __init__(self, size: int) -> None:
        self.size = size
        self.fd: Optional[int] = None

    def __enter__(self) -> int:
        self.open()

        assert self.fd is not None
        return self.fd

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        self.close()

    def open(self) -> None:
        """Open an anonymous file

        Opens the anonymous file and sets the ``fd`` property to the file
        descriptor that has been opened.
        """
        if self.fd is not None:
            raise IOError("File is already open")
        self.fd = lib.os_create_anonymous_file(self.size)
        if self.fd < 0:
            raise IOError("Unable to create anonymous file")

    def close(self) -> None:
        """Close the anonymous file

        Closes the file descriptor and sets the ``fd`` property to ``None``.
        Does nothing if the file is not open.
        """
        if self.fd is None:
            return

        os.close(self.fd)
        self.fd = None


def wl_container_of(ptr: ffi.CData, ctype: str, member: str, *, ffi=ffi) -> ffi.CData:
    """
    #define wl_container_of(ptr, sample, member)				\
            (__typeof__(sample))((char *)(ptr) -				\
                                 offsetof(__typeof__(*sample), member))

    :param ptr:
        Pointer to contained object
    :param ctype:
        ctype of container as string
    :param member:
        Member name of contained object in ctype
    :param ffi:
        ffi module to use. The default is pywayland, but this allows the use of this
        macro by other ffi modules that use `wl_list`s.
    """
    return ffi.cast(ctype, ffi.cast("char *", ptr) - ffi.offsetof(ctype, member))  # type: ignore[no-any-return]


def wl_list_for_each(
    ctype: str, head: ffi.CData, member: str, *, ffi=ffi
) -> Iterator[ffi.CData]:
    """
    #define wl_list_for_each(pos, head, member)				\
        for (pos = wl_container_of((head)->next, pos, member);	\
             &pos->member != (head);					\
             pos = wl_container_of(pos->member.next, pos, member))

    :param ctype:
        ctype of container as string
    :param head:
        The struct wl_list
    :param member:
        Member name of struct wl_list in ctype
    :param ffi:
        ffi module to use. The default is pywayland, but this allows the use of this
        macro by other ffi modules that use `wl_list`s.
    """
    pos = wl_container_of(head.next, ctype, member, ffi=ffi)

    while getattr(pos, member) != head:
        yield pos
        pos = wl_container_of(getattr(pos, member).next, ctype, member, ffi=ffi)
